# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._subscription_schedule import SubscriptionSchedule
    from stripe.params._subscription_schedule_amend_params import (
        SubscriptionScheduleAmendParams,
    )
    from stripe.params._subscription_schedule_cancel_params import (
        SubscriptionScheduleCancelParams,
    )
    from stripe.params._subscription_schedule_create_params import (
        SubscriptionScheduleCreateParams,
    )
    from stripe.params._subscription_schedule_list_params import (
        SubscriptionScheduleListParams,
    )
    from stripe.params._subscription_schedule_release_params import (
        SubscriptionScheduleReleaseParams,
    )
    from stripe.params._subscription_schedule_retrieve_params import (
        SubscriptionScheduleRetrieveParams,
    )
    from stripe.params._subscription_schedule_update_params import (
        SubscriptionScheduleUpdateParams,
    )


class SubscriptionScheduleService(StripeService):
    def list(
        self,
        params: Optional["SubscriptionScheduleListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[SubscriptionSchedule]":
        """
        Retrieves the list of your subscription schedules.
        """
        return cast(
            "ListObject[SubscriptionSchedule]",
            self._request(
                "get",
                "/v1/subscription_schedules",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["SubscriptionScheduleListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[SubscriptionSchedule]":
        """
        Retrieves the list of your subscription schedules.
        """
        return cast(
            "ListObject[SubscriptionSchedule]",
            await self._request_async(
                "get",
                "/v1/subscription_schedules",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["SubscriptionScheduleCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Creates a new subscription schedule object. Each customer can have up to 500 active or scheduled subscriptions.
        """
        return cast(
            "SubscriptionSchedule",
            self._request(
                "post",
                "/v1/subscription_schedules",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["SubscriptionScheduleCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Creates a new subscription schedule object. Each customer can have up to 500 active or scheduled subscriptions.
        """
        return cast(
            "SubscriptionSchedule",
            await self._request_async(
                "post",
                "/v1/subscription_schedules",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        schedule: str,
        params: Optional["SubscriptionScheduleRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Retrieves the details of an existing subscription schedule. You only need to supply the unique subscription schedule identifier that was returned upon subscription schedule creation.
        """
        return cast(
            "SubscriptionSchedule",
            self._request(
                "get",
                "/v1/subscription_schedules/{schedule}".format(
                    schedule=sanitize_id(schedule),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        schedule: str,
        params: Optional["SubscriptionScheduleRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Retrieves the details of an existing subscription schedule. You only need to supply the unique subscription schedule identifier that was returned upon subscription schedule creation.
        """
        return cast(
            "SubscriptionSchedule",
            await self._request_async(
                "get",
                "/v1/subscription_schedules/{schedule}".format(
                    schedule=sanitize_id(schedule),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        schedule: str,
        params: Optional["SubscriptionScheduleUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Updates an existing subscription schedule.
        """
        return cast(
            "SubscriptionSchedule",
            self._request(
                "post",
                "/v1/subscription_schedules/{schedule}".format(
                    schedule=sanitize_id(schedule),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        schedule: str,
        params: Optional["SubscriptionScheduleUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Updates an existing subscription schedule.
        """
        return cast(
            "SubscriptionSchedule",
            await self._request_async(
                "post",
                "/v1/subscription_schedules/{schedule}".format(
                    schedule=sanitize_id(schedule),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def amend(
        self,
        schedule: str,
        params: Optional["SubscriptionScheduleAmendParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Amends an existing subscription schedule.
        """
        return cast(
            "SubscriptionSchedule",
            self._request(
                "post",
                "/v1/subscription_schedules/{schedule}/amend".format(
                    schedule=sanitize_id(schedule),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def amend_async(
        self,
        schedule: str,
        params: Optional["SubscriptionScheduleAmendParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Amends an existing subscription schedule.
        """
        return cast(
            "SubscriptionSchedule",
            await self._request_async(
                "post",
                "/v1/subscription_schedules/{schedule}/amend".format(
                    schedule=sanitize_id(schedule),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        schedule: str,
        params: Optional["SubscriptionScheduleCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Cancels a subscription schedule and its associated subscription immediately (if the subscription schedule has an active subscription). A subscription schedule can only be canceled if its status is not_started or active.
        """
        return cast(
            "SubscriptionSchedule",
            self._request(
                "post",
                "/v1/subscription_schedules/{schedule}/cancel".format(
                    schedule=sanitize_id(schedule),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        schedule: str,
        params: Optional["SubscriptionScheduleCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Cancels a subscription schedule and its associated subscription immediately (if the subscription schedule has an active subscription). A subscription schedule can only be canceled if its status is not_started or active.
        """
        return cast(
            "SubscriptionSchedule",
            await self._request_async(
                "post",
                "/v1/subscription_schedules/{schedule}/cancel".format(
                    schedule=sanitize_id(schedule),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def release(
        self,
        schedule: str,
        params: Optional["SubscriptionScheduleReleaseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Releases the subscription schedule immediately, which will stop scheduling of its phases, but leave any existing subscription in place. A schedule can only be released if its status is not_started or active. If the subscription schedule is currently associated with a subscription, releasing it will remove its subscription property and set the subscription's ID to the released_subscription property.
        """
        return cast(
            "SubscriptionSchedule",
            self._request(
                "post",
                "/v1/subscription_schedules/{schedule}/release".format(
                    schedule=sanitize_id(schedule),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def release_async(
        self,
        schedule: str,
        params: Optional["SubscriptionScheduleReleaseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SubscriptionSchedule":
        """
        Releases the subscription schedule immediately, which will stop scheduling of its phases, but leave any existing subscription in place. A schedule can only be released if its status is not_started or active. If the subscription schedule is currently associated with a subscription, releasing it will remove its subscription property and set the subscription's ID to the released_subscription property.
        """
        return cast(
            "SubscriptionSchedule",
            await self._request_async(
                "post",
                "/v1/subscription_schedules/{schedule}/release".format(
                    schedule=sanitize_id(schedule),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.identity._verification_report import VerificationReport
    from stripe.identity._verification_session import VerificationSession
    from stripe.params.identity._blocklist_entry_create_params import (
        BlocklistEntryCreateParams,
    )
    from stripe.params.identity._blocklist_entry_disable_params import (
        BlocklistEntryDisableParams,
    )
    from stripe.params.identity._blocklist_entry_list_params import (
        BlocklistEntryListParams,
    )
    from stripe.params.identity._blocklist_entry_retrieve_params import (
        BlocklistEntryRetrieveParams,
    )


class BlocklistEntry(
    CreateableAPIResource["BlocklistEntry"],
    ListableAPIResource["BlocklistEntry"],
):
    """
    A BlocklistEntry represents an entry in our identity verification blocklist.
    It helps prevent fraudulent users from repeatedly attempting verification with similar information.
    When you create a BlocklistEntry, we store data from a specified VerificationReport,
    such as document details or facial biometrics.
    This allows us to compare future verification attempts against these entries.
    If a match is found, we categorize the new verification as unverified.

    To learn more, see [Identity Verification Blocklist](https://stripe.com/docs/identity/review-tools#block-list)
    """

    OBJECT_NAME: ClassVar[Literal["identity.blocklist_entry"]] = (
        "identity.blocklist_entry"
    )
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    disabled_at: Optional[int]
    """
    Time at which you disabled the BlocklistEntry. Measured in seconds since the Unix epoch.
    """
    expires_at: Optional[int]
    """
    Time at which the BlocklistEntry expires. Measured in seconds since the Unix epoch.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["identity.blocklist_entry"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    status: Literal["active", "disabled", "redacted"]
    """
    The current status of the BlocklistEntry.
    """
    type: Literal["document", "selfie"]
    """
    The type of BlocklistEntry.
    """
    verification_report: Optional[ExpandableField["VerificationReport"]]
    """
    The verification report the BlocklistEntry was created from.
    """
    verification_session: Optional[ExpandableField["VerificationSession"]]
    """
    The verification session the BlocklistEntry was created from.
    """

    @classmethod
    def create(
        cls, **params: Unpack["BlocklistEntryCreateParams"]
    ) -> "BlocklistEntry":
        """
        Creates a BlocklistEntry object from a verification report.

        A blocklist entry prevents future identity verifications that match the same identity information.
        You can create blocklist entries from verification reports that contain document extracted data
        or a selfie.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#add-a-blocklist-entry)
        """
        return cast(
            "BlocklistEntry",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["BlocklistEntryCreateParams"]
    ) -> "BlocklistEntry":
        """
        Creates a BlocklistEntry object from a verification report.

        A blocklist entry prevents future identity verifications that match the same identity information.
        You can create blocklist entries from verification reports that contain document extracted data
        or a selfie.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#add-a-blocklist-entry)
        """
        return cast(
            "BlocklistEntry",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_disable(
        cls, id: str, **params: Unpack["BlocklistEntryDisableParams"]
    ) -> "BlocklistEntry":
        """
        Disables a BlocklistEntry object.

        After a BlocklistEntry is disabled, it will no longer block future verifications that match
        the same information. This action is irreversible. To re-enable it, a new BlocklistEntry
        must be created using the same verification report.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#disable-a-blocklist-entry)
        """
        return cast(
            "BlocklistEntry",
            cls._static_request(
                "post",
                "/v1/identity/blocklist_entries/{id}/disable".format(
                    id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def disable(
        id: str, **params: Unpack["BlocklistEntryDisableParams"]
    ) -> "BlocklistEntry":
        """
        Disables a BlocklistEntry object.

        After a BlocklistEntry is disabled, it will no longer block future verifications that match
        the same information. This action is irreversible. To re-enable it, a new BlocklistEntry
        must be created using the same verification report.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#disable-a-blocklist-entry)
        """
        ...

    @overload
    def disable(
        self, **params: Unpack["BlocklistEntryDisableParams"]
    ) -> "BlocklistEntry":
        """
        Disables a BlocklistEntry object.

        After a BlocklistEntry is disabled, it will no longer block future verifications that match
        the same information. This action is irreversible. To re-enable it, a new BlocklistEntry
        must be created using the same verification report.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#disable-a-blocklist-entry)
        """
        ...

    @class_method_variant("_cls_disable")
    def disable(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["BlocklistEntryDisableParams"]
    ) -> "BlocklistEntry":
        """
        Disables a BlocklistEntry object.

        After a BlocklistEntry is disabled, it will no longer block future verifications that match
        the same information. This action is irreversible. To re-enable it, a new BlocklistEntry
        must be created using the same verification report.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#disable-a-blocklist-entry)
        """
        return cast(
            "BlocklistEntry",
            self._request(
                "post",
                "/v1/identity/blocklist_entries/{id}/disable".format(
                    id=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_disable_async(
        cls, id: str, **params: Unpack["BlocklistEntryDisableParams"]
    ) -> "BlocklistEntry":
        """
        Disables a BlocklistEntry object.

        After a BlocklistEntry is disabled, it will no longer block future verifications that match
        the same information. This action is irreversible. To re-enable it, a new BlocklistEntry
        must be created using the same verification report.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#disable-a-blocklist-entry)
        """
        return cast(
            "BlocklistEntry",
            await cls._static_request_async(
                "post",
                "/v1/identity/blocklist_entries/{id}/disable".format(
                    id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def disable_async(
        id: str, **params: Unpack["BlocklistEntryDisableParams"]
    ) -> "BlocklistEntry":
        """
        Disables a BlocklistEntry object.

        After a BlocklistEntry is disabled, it will no longer block future verifications that match
        the same information. This action is irreversible. To re-enable it, a new BlocklistEntry
        must be created using the same verification report.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#disable-a-blocklist-entry)
        """
        ...

    @overload
    async def disable_async(
        self, **params: Unpack["BlocklistEntryDisableParams"]
    ) -> "BlocklistEntry":
        """
        Disables a BlocklistEntry object.

        After a BlocklistEntry is disabled, it will no longer block future verifications that match
        the same information. This action is irreversible. To re-enable it, a new BlocklistEntry
        must be created using the same verification report.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#disable-a-blocklist-entry)
        """
        ...

    @class_method_variant("_cls_disable_async")
    async def disable_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["BlocklistEntryDisableParams"]
    ) -> "BlocklistEntry":
        """
        Disables a BlocklistEntry object.

        After a BlocklistEntry is disabled, it will no longer block future verifications that match
        the same information. This action is irreversible. To re-enable it, a new BlocklistEntry
        must be created using the same verification report.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#disable-a-blocklist-entry)
        """
        return cast(
            "BlocklistEntry",
            await self._request_async(
                "post",
                "/v1/identity/blocklist_entries/{id}/disable".format(
                    id=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["BlocklistEntryListParams"]
    ) -> ListObject["BlocklistEntry"]:
        """
        Returns a list of BlocklistEntry objects associated with your account.

        The blocklist entries are returned sorted by creation date, with the most recently created
        entries appearing first.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#block-list)
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["BlocklistEntryListParams"]
    ) -> ListObject["BlocklistEntry"]:
        """
        Returns a list of BlocklistEntry objects associated with your account.

        The blocklist entries are returned sorted by creation date, with the most recently created
        entries appearing first.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#block-list)
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["BlocklistEntryRetrieveParams"]
    ) -> "BlocklistEntry":
        """
        Retrieves a BlocklistEntry object by its identifier.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#block-list)
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["BlocklistEntryRetrieveParams"]
    ) -> "BlocklistEntry":
        """
        Retrieves a BlocklistEntry object by its identifier.

        Related guide: [Identity Verification Blocklist](https://docs.stripe.com/docs/identity/review-tools#block-list)
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def class_url(cls):
        return "/v1/identity/blocklist_entries"



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired


class ProgramCreateParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    is_default: NotRequired[bool]
    """
    If true, makes the specified program the default for the given account.
    """
    platform_program: str
    """
    The program to use as the parent for the new program to create.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class FinancialAccountFeaturesUpdateParams(TypedDict):
    card_issuing: NotRequired[
        "FinancialAccountFeaturesUpdateParamsCardIssuing"
    ]
    """
    Encodes the FinancialAccount's ability to be used with the Issuing product, including attaching cards to and drawing funds from the FinancialAccount.
    """
    deposit_insurance: NotRequired[
        "FinancialAccountFeaturesUpdateParamsDepositInsurance"
    ]
    """
    Represents whether this FinancialAccount is eligible for deposit insurance. Various factors determine the insurance amount.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    financial_addresses: NotRequired[
        "FinancialAccountFeaturesUpdateParamsFinancialAddresses"
    ]
    """
    Contains Features that add FinancialAddresses to the FinancialAccount.
    """
    inbound_transfers: NotRequired[
        "FinancialAccountFeaturesUpdateParamsInboundTransfers"
    ]
    """
    Contains settings related to adding funds to a FinancialAccount from another Account with the same owner.
    """
    intra_stripe_flows: NotRequired[
        "FinancialAccountFeaturesUpdateParamsIntraStripeFlows"
    ]
    """
    Represents the ability for the FinancialAccount to send money to, or receive money from other FinancialAccounts (for example, via OutboundPayment).
    """
    outbound_payments: NotRequired[
        "FinancialAccountFeaturesUpdateParamsOutboundPayments"
    ]
    """
    Includes Features related to initiating money movement out of the FinancialAccount to someone else's bucket of money.
    """
    outbound_transfers: NotRequired[
        "FinancialAccountFeaturesUpdateParamsOutboundTransfers"
    ]
    """
    Contains a Feature and settings related to moving money out of the FinancialAccount into another Account with the same owner.
    """


class FinancialAccountFeaturesUpdateParamsCardIssuing(TypedDict):
    requested: bool
    """
    Whether the FinancialAccount should have the Feature.
    """


class FinancialAccountFeaturesUpdateParamsDepositInsurance(TypedDict):
    requested: bool
    """
    Whether the FinancialAccount should have the Feature.
    """


class FinancialAccountFeaturesUpdateParamsFinancialAddresses(TypedDict):
    aba: NotRequired[
        "FinancialAccountFeaturesUpdateParamsFinancialAddressesAba"
    ]
    """
    Adds an ABA FinancialAddress to the FinancialAccount.
    """


class FinancialAccountFeaturesUpdateParamsFinancialAddressesAba(TypedDict):
    bank: NotRequired[Literal["evolve", "fifth_third", "goldman_sachs"]]
    """
    Requested bank partner
    """
    requested: bool
    """
    Whether the FinancialAccount should have the Feature.
    """


class FinancialAccountFeaturesUpdateParamsInboundTransfers(TypedDict):
    ach: NotRequired["FinancialAccountFeaturesUpdateParamsInboundTransfersAch"]
    """
    Enables ACH Debits via the InboundTransfers API.
    """


class FinancialAccountFeaturesUpdateParamsInboundTransfersAch(TypedDict):
    requested: bool
    """
    Whether the FinancialAccount should have the Feature.
    """


class FinancialAccountFeaturesUpdateParamsIntraStripeFlows(TypedDict):
    requested: bool
    """
    Whether the FinancialAccount should have the Feature.
    """


class FinancialAccountFeaturesUpdateParamsOutboundPayments(TypedDict):
    ach: NotRequired["FinancialAccountFeaturesUpdateParamsOutboundPaymentsAch"]
    """
    Enables ACH transfers via the OutboundPayments API.
    """
    us_domestic_wire: NotRequired[
        "FinancialAccountFeaturesUpdateParamsOutboundPaymentsUsDomesticWire"
    ]
    """
    Enables US domestic wire transfers via the OutboundPayments API.
    """


class FinancialAccountFeaturesUpdateParamsOutboundPaymentsAch(TypedDict):
    requested: bool
    """
    Whether the FinancialAccount should have the Feature.
    """


class FinancialAccountFeaturesUpdateParamsOutboundPaymentsUsDomesticWire(
    TypedDict,
):
    requested: bool
    """
    Whether the FinancialAccount should have the Feature.
    """


class FinancialAccountFeaturesUpdateParamsOutboundTransfers(TypedDict):
    ach: NotRequired[
        "FinancialAccountFeaturesUpdateParamsOutboundTransfersAch"
    ]
    """
    Enables ACH transfers via the OutboundTransfers API.
    """
    us_domestic_wire: NotRequired[
        "FinancialAccountFeaturesUpdateParamsOutboundTransfersUsDomesticWire"
    ]
    """
    Enables US domestic wire transfers via the OutboundTransfers API.
    """


class FinancialAccountFeaturesUpdateParamsOutboundTransfersAch(TypedDict):
    requested: bool
    """
    Whether the FinancialAccount should have the Feature.
    """


class FinancialAccountFeaturesUpdateParamsOutboundTransfersUsDomesticWire(
    TypedDict,
):
    requested: bool
    """
    Whether the FinancialAccount should have the Feature.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Dict, Optional
from typing_extensions import Literal


class LicenseFeeSubscription(StripeObject):
    OBJECT_NAME: ClassVar[Literal["v2.billing.license_fee_subscription"]] = (
        "v2.billing.license_fee_subscription"
    )
    billing_cadence: str
    """
    The ID of the Billing Cadence.
    """
    created: str
    """
    Timestamp of when the object was created.
    """
    id: str
    """
    Unique identifier for the object.
    """
    license_fee: str
    """
    The ID of the License Fee.
    """
    license_fee_version: str
    """
    The ID of the License Fee Version.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[Dict[str, str]]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["v2.billing.license_fee_subscription"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    quantity: int
    """
    Quantity of the License Fee subscribed to.
    """
    test_clock: Optional[str]
    """
    The ID of the Test Clock, if any.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class AccountLinkCreateParams(TypedDict):
    account: str
    """
    The ID of the Account to create link for.
    """
    use_case: "AccountLinkCreateParamsUseCase"
    """
    The use case of the AccountLink.
    """


class AccountLinkCreateParamsUseCase(TypedDict):
    type: Literal["account_onboarding", "account_update"]
    """
    Open Enum. The type of AccountLink the user is requesting.
    """
    account_onboarding: NotRequired[
        "AccountLinkCreateParamsUseCaseAccountOnboarding"
    ]
    """
    Indicates that the AccountLink provided should onboard an account.
    """
    account_update: NotRequired["AccountLinkCreateParamsUseCaseAccountUpdate"]
    """
    Indicates that the AccountLink provided should update a previously onboarded account.
    """


class AccountLinkCreateParamsUseCaseAccountOnboarding(TypedDict):
    collection_options: NotRequired[
        "AccountLinkCreateParamsUseCaseAccountOnboardingCollectionOptions"
    ]
    """
    Specifies the requirements that Stripe collects from v2/core/accounts in the Onboarding flow.
    """
    configurations: List[
        Literal["customer", "merchant", "recipient", "storer"]
    ]
    """
    Open Enum. A v2/core/account can be configured to enable certain functionality. The configuration param targets the v2/core/account_link to collect information for the specified v2/core/account configuration/s.
    """
    refresh_url: str
    """
    The URL the user will be redirected to if the AccountLink is expired, has been used, or is otherwise invalid. The URL you specify should attempt to generate a new AccountLink with the same parameters used to create the original AccountLink, then redirect the user to the new AccountLink's URL so they can continue the flow. If a new AccountLink cannot be generated or the redirect fails you should display a useful error to the user. Please make sure to implement authentication before redirecting the user in case this URL is leaked to a third party.
    """
    return_url: NotRequired[str]
    """
    The URL that the user will be redirected to upon completing the linked flow.
    """


class AccountLinkCreateParamsUseCaseAccountOnboardingCollectionOptions(
    TypedDict,
):
    fields: NotRequired[Literal["currently_due", "eventually_due"]]
    """
    Specifies whether the platform collects only currently_due requirements (`currently_due`) or both currently_due and eventually_due requirements (`eventually_due`). If you don't specify collection_options, the default value is currently_due.
    """
    future_requirements: NotRequired[Literal["include", "omit"]]
    """
    Specifies whether the platform collects future_requirements in addition to requirements in Connect Onboarding. The default value is `omit`.
    """


class AccountLinkCreateParamsUseCaseAccountUpdate(TypedDict):
    collection_options: NotRequired[
        "AccountLinkCreateParamsUseCaseAccountUpdateCollectionOptions"
    ]
    """
    Specifies the requirements that Stripe collects from v2/core/accounts in the Onboarding flow.
    """
    configurations: List[
        Literal["customer", "merchant", "recipient", "storer"]
    ]
    """
    Open Enum. A v2/account can be configured to enable certain functionality. The configuration param targets the v2/account_link to collect information for the specified v2/account configuration/s.
    """
    refresh_url: str
    """
    The URL the user will be redirected to if the AccountLink is expired, has been used, or is otherwise invalid. The URL you specify should attempt to generate a new AccountLink with the same parameters used to create the original AccountLink, then redirect the user to the new AccountLink's URL so they can continue the flow. If a new AccountLink cannot be generated or the redirect fails you should display a useful error to the user. Please make sure to implement authentication before redirecting the user in case this URL is leaked to a third party.
    """
    return_url: NotRequired[str]
    """
    The URL that the user will be redirected to upon completing the linked flow.
    """


class AccountLinkCreateParamsUseCaseAccountUpdateCollectionOptions(TypedDict):
    fields: NotRequired[Literal["currently_due", "eventually_due"]]
    """
    Specifies whether the platform collects only currently_due requirements (`currently_due`) or both currently_due and eventually_due requirements (`eventually_due`). If you don't specify collection_options, the default value is currently_due.
    """
    future_requirements: NotRequired[Literal["include", "omit"]]
    """
    Specifies whether the platform collects future_requirements in addition to requirements in Connect Onboarding. The default value is `omit`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.tax._association import Association as Association
    from stripe.tax._association_service import (
        AssociationService as AssociationService,
    )
    from stripe.tax._calculation import Calculation as Calculation
    from stripe.tax._calculation_line_item import (
        CalculationLineItem as CalculationLineItem,
    )
    from stripe.tax._calculation_line_item_service import (
        CalculationLineItemService as CalculationLineItemService,
    )
    from stripe.tax._calculation_service import (
        CalculationService as CalculationService,
    )
    from stripe.tax._form import Form as Form
    from stripe.tax._form_service import FormService as FormService
    from stripe.tax._registration import Registration as Registration
    from stripe.tax._registration_service import (
        RegistrationService as RegistrationService,
    )
    from stripe.tax._settings import Settings as Settings
    from stripe.tax._settings_service import SettingsService as SettingsService
    from stripe.tax._transaction import Transaction as Transaction
    from stripe.tax._transaction_line_item import (
        TransactionLineItem as TransactionLineItem,
    )
    from stripe.tax._transaction_line_item_service import (
        TransactionLineItemService as TransactionLineItemService,
    )
    from stripe.tax._transaction_service import (
        TransactionService as TransactionService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "Association": ("stripe.tax._association", False),
    "AssociationService": ("stripe.tax._association_service", False),
    "Calculation": ("stripe.tax._calculation", False),
    "CalculationLineItem": ("stripe.tax._calculation_line_item", False),
    "CalculationLineItemService": (
        "stripe.tax._calculation_line_item_service",
        False,
    ),
    "CalculationService": ("stripe.tax._calculation_service", False),
    "Form": ("stripe.tax._form", False),
    "FormService": ("stripe.tax._form_service", False),
    "Registration": ("stripe.tax._registration", False),
    "RegistrationService": ("stripe.tax._registration_service", False),
    "Settings": ("stripe.tax._settings", False),
    "SettingsService": ("stripe.tax._settings_service", False),
    "Transaction": ("stripe.tax._transaction", False),
    "TransactionLineItem": ("stripe.tax._transaction_line_item", False),
    "TransactionLineItemService": (
        "stripe.tax._transaction_line_item_service",
        False,
    ),
    "TransactionService": ("stripe.tax._transaction_service", False),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class FinancialAccountListParams(TypedDict):
    limit: NotRequired[int]
    """
    The page limit.
    """
    status: NotRequired[Literal["closed", "open", "pending"]]
    """
    The status of the FinancialAccount to filter by. By default, closed FinancialAccounts are not returned.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._credit_note import CreditNote
    from stripe._credit_note_line_item_service import CreditNoteLineItemService
    from stripe._credit_note_preview_lines_service import (
        CreditNotePreviewLinesService,
    )
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._credit_note_create_params import CreditNoteCreateParams
    from stripe.params._credit_note_list_params import CreditNoteListParams
    from stripe.params._credit_note_preview_params import (
        CreditNotePreviewParams,
    )
    from stripe.params._credit_note_retrieve_params import (
        CreditNoteRetrieveParams,
    )
    from stripe.params._credit_note_update_params import CreditNoteUpdateParams
    from stripe.params._credit_note_void_credit_note_params import (
        CreditNoteVoidCreditNoteParams,
    )

_subservices = {
    "line_items": [
        "stripe._credit_note_line_item_service",
        "CreditNoteLineItemService",
    ],
    "preview_lines": [
        "stripe._credit_note_preview_lines_service",
        "CreditNotePreviewLinesService",
    ],
}


class CreditNoteService(StripeService):
    line_items: "CreditNoteLineItemService"
    preview_lines: "CreditNotePreviewLinesService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()

    def list(
        self,
        params: Optional["CreditNoteListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[CreditNote]":
        """
        Returns a list of credit notes.
        """
        return cast(
            "ListObject[CreditNote]",
            self._request(
                "get",
                "/v1/credit_notes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["CreditNoteListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[CreditNote]":
        """
        Returns a list of credit notes.
        """
        return cast(
            "ListObject[CreditNote]",
            await self._request_async(
                "get",
                "/v1/credit_notes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "CreditNoteCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CreditNote":
        """
        Issue a credit note to adjust the amount of a finalized invoice. A credit note will first reduce the invoice's amount_remaining (and amount_due), but not below zero.
        This amount is indicated by the credit note's pre_payment_amount. The excess amount is indicated by post_payment_amount, and it can result in any combination of the following:


        Refunds: create a new refund (using refund_amount) or link existing refunds (using refunds).
        Customer balance credit: credit the customer's balance (using credit_amount) which will be automatically applied to their next invoice when it's finalized.
        Outside of Stripe credit: record the amount that is or will be credited outside of Stripe (using out_of_band_amount).


        The sum of refunds, customer balance credits, and outside of Stripe credits must equal the post_payment_amount.

        You may issue multiple credit notes for an invoice. Each credit note may increment the invoice's pre_payment_credit_notes_amount,
        post_payment_credit_notes_amount, or both, depending on the invoice's amount_remaining at the time of credit note creation.
        """
        return cast(
            "CreditNote",
            self._request(
                "post",
                "/v1/credit_notes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "CreditNoteCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CreditNote":
        """
        Issue a credit note to adjust the amount of a finalized invoice. A credit note will first reduce the invoice's amount_remaining (and amount_due), but not below zero.
        This amount is indicated by the credit note's pre_payment_amount. The excess amount is indicated by post_payment_amount, and it can result in any combination of the following:


        Refunds: create a new refund (using refund_amount) or link existing refunds (using refunds).
        Customer balance credit: credit the customer's balance (using credit_amount) which will be automatically applied to their next invoice when it's finalized.
        Outside of Stripe credit: record the amount that is or will be credited outside of Stripe (using out_of_band_amount).


        The sum of refunds, customer balance credits, and outside of Stripe credits must equal the post_payment_amount.

        You may issue multiple credit notes for an invoice. Each credit note may increment the invoice's pre_payment_credit_notes_amount,
        post_payment_credit_notes_amount, or both, depending on the invoice's amount_remaining at the time of credit note creation.
        """
        return cast(
            "CreditNote",
            await self._request_async(
                "post",
                "/v1/credit_notes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["CreditNoteRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CreditNote":
        """
        Retrieves the credit note object with the given identifier.
        """
        return cast(
            "CreditNote",
            self._request(
                "get",
                "/v1/credit_notes/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["CreditNoteRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CreditNote":
        """
        Retrieves the credit note object with the given identifier.
        """
        return cast(
            "CreditNote",
            await self._request_async(
                "get",
                "/v1/credit_notes/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["CreditNoteUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CreditNote":
        """
        Updates an existing credit note.
        """
        return cast(
            "CreditNote",
            self._request(
                "post",
                "/v1/credit_notes/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["CreditNoteUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CreditNote":
        """
        Updates an existing credit note.
        """
        return cast(
            "CreditNote",
            await self._request_async(
                "post",
                "/v1/credit_notes/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def preview(
        self,
        params: "CreditNotePreviewParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CreditNote":
        """
        Get a preview of a credit note without creating it.
        """
        return cast(
            "CreditNote",
            self._request(
                "get",
                "/v1/credit_notes/preview",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def preview_async(
        self,
        params: "CreditNotePreviewParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CreditNote":
        """
        Get a preview of a credit note without creating it.
        """
        return cast(
            "CreditNote",
            await self._request_async(
                "get",
                "/v1/credit_notes/preview",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def void_credit_note(
        self,
        id: str,
        params: Optional["CreditNoteVoidCreditNoteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CreditNote":
        """
        Marks a credit note as void. Learn more about [voiding credit notes](https://docs.stripe.com/docs/billing/invoices/credit-notes#voiding).
        """
        return cast(
            "CreditNote",
            self._request(
                "post",
                "/v1/credit_notes/{id}/void".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def void_credit_note_async(
        self,
        id: str,
        params: Optional["CreditNoteVoidCreditNoteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CreditNote":
        """
        Marks a credit note as void. Learn more about [voiding credit notes](https://docs.stripe.com/docs/billing/invoices/credit-notes#voiding).
        """
        return cast(
            "CreditNote",
            await self._request_async(
                "post",
                "/v1/credit_notes/{id}/void".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class ProductUpdateParams(TypedDict):
    active: NotRequired[bool]
    """
    Whether the product is available for purchase.
    """
    default_price: NotRequired[str]
    """
    The ID of the [Price](https://stripe.com/docs/api/prices) object that is the default price for this product.
    """
    description: NotRequired["Literal['']|str"]
    """
    The product's description, meant to be displayable to the customer. Use this field to optionally store a long form explanation of the product being sold for your own rendering purposes.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    images: NotRequired["Literal['']|List[str]"]
    """
    A list of up to 8 URLs of images for this product, meant to be displayable to the customer.
    """
    marketing_features: NotRequired[
        "Literal['']|List[ProductUpdateParamsMarketingFeature]"
    ]
    """
    A list of up to 15 marketing features for this product. These are displayed in [pricing tables](https://stripe.com/docs/payments/checkout/pricing-table).
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    name: NotRequired[str]
    """
    The product's name, meant to be displayable to the customer.
    """
    package_dimensions: NotRequired[
        "Literal['']|ProductUpdateParamsPackageDimensions"
    ]
    """
    The dimensions of this product for shipping purposes.
    """
    shippable: NotRequired[bool]
    """
    Whether this product is shipped (i.e., physical goods).
    """
    statement_descriptor: NotRequired[str]
    """
    An arbitrary string to be displayed on your customer's credit card or bank statement. While most banks display this information consistently, some may display it incorrectly or not at all.

    This may be up to 22 characters. The statement description may not include `<`, `>`, `\\`, `"`, `'` characters, and will appear on your customer's statement in capital letters. Non-ASCII characters are automatically stripped.
     It must contain at least one letter. May only be set if `type=service`. Only used for subscription payments.
    """
    tax_code: NotRequired["Literal['']|str"]
    """
    A [tax code](https://stripe.com/docs/tax/tax-categories) ID.
    """
    unit_label: NotRequired["Literal['']|str"]
    """
    A label that represents units of this product. When set, this will be included in customers' receipts, invoices, Checkout, and the customer portal. May only be set if `type=service`.
    """
    url: NotRequired["Literal['']|str"]
    """
    A URL of a publicly-accessible webpage for this product.
    """


class ProductUpdateParamsMarketingFeature(TypedDict):
    name: str
    """
    The marketing feature name. Up to 80 characters long.
    """


class ProductUpdateParamsPackageDimensions(TypedDict):
    height: float
    """
    Height, in inches. Maximum precision is 2 decimal places.
    """
    length: float
    """
    Length, in inches. Maximum precision is 2 decimal places.
    """
    weight: float
    """
    Weight, in ounces. Maximum precision is 2 decimal places.
    """
    width: float
    """
    Width, in inches. Maximum precision is 2 decimal places.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class CreditNotePreviewLinesListParams(TypedDict):
    amount: NotRequired[int]
    """
    The integer amount in cents (or local equivalent) representing the total amount of the credit note. One of `amount`, `lines`, or `shipping_cost` must be provided.
    """
    credit_amount: NotRequired[int]
    """
    The integer amount in cents (or local equivalent) representing the amount to credit the customer's balance, which will be automatically applied to their next invoice.
    """
    effective_at: NotRequired[int]
    """
    The date when this credit note is in effect. Same as `created` unless overwritten. When defined, this value replaces the system-generated 'Date of issue' printed on the credit note PDF.
    """
    email_type: NotRequired[Literal["credit_note", "none"]]
    """
    Type of email to send to the customer, one of `credit_note` or `none` and the default is `credit_note`.
    """
    ending_before: NotRequired[str]
    """
    A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    invoice: str
    """
    ID of the invoice.
    """
    limit: NotRequired[int]
    """
    A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
    """
    lines: NotRequired[List["CreditNotePreviewLinesListParamsLine"]]
    """
    Line items that make up the credit note. One of `amount`, `lines`, or `shipping_cost` must be provided.
    """
    memo: NotRequired[str]
    """
    The credit note's memo appears on the credit note PDF.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    out_of_band_amount: NotRequired[int]
    """
    The integer amount in cents (or local equivalent) representing the amount that is credited outside of Stripe.
    """
    reason: NotRequired[
        Literal[
            "duplicate", "fraudulent", "order_change", "product_unsatisfactory"
        ]
    ]
    """
    Reason for issuing this credit note, one of `duplicate`, `fraudulent`, `order_change`, or `product_unsatisfactory`
    """
    refund_amount: NotRequired[int]
    """
    The integer amount in cents (or local equivalent) representing the amount to refund. If set, a refund will be created for the charge associated with the invoice.
    """
    refunds: NotRequired[List["CreditNotePreviewLinesListParamsRefund"]]
    """
    Refunds to link to this credit note.
    """
    shipping_cost: NotRequired["CreditNotePreviewLinesListParamsShippingCost"]
    """
    When shipping_cost contains the shipping_rate from the invoice, the shipping_cost is included in the credit note. One of `amount`, `lines`, or `shipping_cost` must be provided.
    """
    starting_after: NotRequired[str]
    """
    A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
    """


class CreditNotePreviewLinesListParamsLine(TypedDict):
    amount: NotRequired[int]
    """
    The line item amount to credit. Only valid when `type` is `invoice_line_item`. If invoice is set up with `automatic_tax[enabled]=true`, this amount is tax exclusive
    """
    description: NotRequired[str]
    """
    The description of the credit note line item. Only valid when the `type` is `custom_line_item`.
    """
    invoice_line_item: NotRequired[str]
    """
    The invoice line item to credit. Only valid when the `type` is `invoice_line_item`.
    """
    quantity: NotRequired[int]
    """
    The line item quantity to credit.
    """
    tax_amounts: NotRequired[
        "Literal['']|List[CreditNotePreviewLinesListParamsLineTaxAmount]"
    ]
    """
    A list of up to 10 tax amounts for the credit note line item. Cannot be mixed with `tax_rates`.
    """
    tax_rates: NotRequired["Literal['']|List[str]"]
    """
    The tax rates which apply to the credit note line item. Only valid when the `type` is `custom_line_item` and cannot be mixed with `tax_amounts`.
    """
    type: Literal["custom_line_item", "invoice_line_item"]
    """
    Type of the credit note line item, one of `invoice_line_item` or `custom_line_item`
    """
    unit_amount: NotRequired[int]
    """
    The integer unit amount in cents (or local equivalent) of the credit note line item. This `unit_amount` will be multiplied by the quantity to get the full amount to credit for this line item. Only valid when `type` is `custom_line_item`.
    """
    unit_amount_decimal: NotRequired[str]
    """
    Same as `unit_amount`, but accepts a decimal value in cents (or local equivalent) with at most 12 decimal places. Only one of `unit_amount` and `unit_amount_decimal` can be set.
    """


class CreditNotePreviewLinesListParamsLineTaxAmount(TypedDict):
    amount: int
    """
    The amount, in cents (or local equivalent), of the tax.
    """
    tax_rate: str
    """
    The id of the tax rate for this tax amount. The tax rate must have been automatically created by Stripe.
    """
    taxable_amount: int
    """
    The amount on which tax is calculated, in cents (or local equivalent).
    """


class CreditNotePreviewLinesListParamsRefund(TypedDict):
    amount_refunded: NotRequired[int]
    """
    Amount of the refund that applies to this credit note, in cents (or local equivalent). Defaults to the entire refund amount.
    """
    payment_record_refund: NotRequired[
        "CreditNotePreviewLinesListParamsRefundPaymentRecordRefund"
    ]
    """
    The PaymentRecord refund details to link to this credit note. Required when `type` is `payment_record_refund`.
    """
    refund: NotRequired[str]
    """
    ID of an existing refund to link this credit note to. Required when `type` is `refund`.
    """
    type: NotRequired[Literal["payment_record_refund", "refund"]]
    """
    Type of the refund, one of `refund` or `payment_record_refund`. Defaults to `refund`.
    """


class CreditNotePreviewLinesListParamsRefundPaymentRecordRefund(TypedDict):
    payment_record: str
    """
    The ID of the PaymentRecord with the refund to link to this credit note.
    """
    refund_group: str
    """
    The PaymentRecord refund group to link to this credit note. For refunds processed off-Stripe, this will correspond to the `processor_details.custom.refund_reference` field provided when reporting the refund on the PaymentRecord.
    """


class CreditNotePreviewLinesListParamsShippingCost(TypedDict):
    shipping_rate: NotRequired[str]
    """
    The ID of the shipping rate to use for this order.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class ClaimableSandbox(StripeObject):
    """
    A claimable sandbox represents a Stripe sandbox that is anonymous.
    When it is created, it can be prefilled with specific metadata, such as email, name, or country.
    Claimable sandboxes can be claimed through a URL. When a user claims a sandbox through this URL,
    it will prompt them to create a new Stripe account. Or, it will allow them to claim this sandbox in their
    existing Stripe account.
    Claimable sandboxes have 60 days to be claimed. After this expiration time has passed,
    if the sandbox is not claimed, it will be deleted.
    """

    OBJECT_NAME: ClassVar[Literal["v2.core.claimable_sandbox"]] = (
        "v2.core.claimable_sandbox"
    )

    class Prefill(StripeObject):
        country: str
        """
        Country in which the account holder resides, or in which the business is legally established.
        Use two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        email: str
        """
        Email that this sandbox is meant to be claimed by. Stripe will
        send an email to this email address before the sandbox expires.
        """
        name: str
        """
        Name for the sandbox.
        """

    class SandboxDetails(StripeObject):
        class ApiKeys(StripeObject):
            mcp: Optional[str]
            """
            Used to communicate with [Stripe's MCP server](https://docs.stripe.com/mcp).
            This allows LLM agents to securely operate on a Stripe account.
            """
            publishable: str
            """
            Publicly accessible in a web or mobile app client-side code.
            """
            secret: str
            """
            Should be stored securely in server-side code (such as an environment variable).
            """

        account: str
        """
        The sandbox's Stripe account ID.
        """
        api_keys: Optional[ApiKeys]
        """
        Keys that can be used to set up an integration for this sandbox and operate on the account. This will be present only in the create response, and will be null in subsequent retrieve responses.
        """
        owner_account: Optional[str]
        """
        The livemode sandbox Stripe account ID. This field is only set if the user activates their sandbox
        and chooses to install your platform's Stripe App in their live account.
        """
        _inner_class_types = {"api_keys": ApiKeys}

    claim_url: Optional[str]
    """
    URL for user to claim sandbox into their existing Stripe account.
    The value will be null if the sandbox status is `claimed` or `expired`.
    """
    claimed_at: Optional[str]
    """
    The timestamp the sandbox was claimed. The value will be null if the sandbox status is not `claimed`.
    """
    created: str
    """
    When the sandbox is created.
    """
    expires_at: Optional[str]
    """
    The timestamp the sandbox will expire. The value will be null if the sandbox is `claimed`.
    """
    id: str
    """
    Unique identifier for the Claimable sandbox.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.core.claimable_sandbox"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    prefill: Prefill
    """
    Values prefilled during the creation of the sandbox. When a user claims the sandbox, they will be able to update these values.
    """
    sandbox_details: SandboxDetails
    """
    Data about the Stripe sandbox object.
    """
    status: Literal["claimed", "expired", "unclaimed"]
    """
    Status of the sandbox. One of `unclaimed`, `expired`, `claimed`.
    """
    _inner_class_types = {
        "prefill": Prefill,
        "sandbox_details": SandboxDetails,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class BalanceTransferCreateParams(RequestOptions):
    amount: int
    """
    A positive integer representing how much to transfer in the smallest currency unit.
    """
    currency: Literal["eur", "gbp", "usd"]
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    destination_balance: "BalanceTransferCreateParamsDestinationBalance"
    """
    The balance to which funds are transferred.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    source_balance: "BalanceTransferCreateParamsSourceBalance"
    """
    The balance from which funds are transferred, including details specific to the balance you choose.
    """


class BalanceTransferCreateParamsDestinationBalance(TypedDict):
    type: Literal["issuing", "payments"]
    """
    Destination balance type to push funds into for the Balance Transfer.
    """


class BalanceTransferCreateParamsSourceBalance(TypedDict):
    allocated_funds: NotRequired[
        "BalanceTransferCreateParamsSourceBalanceAllocatedFunds"
    ]
    type: Literal["allocated_funds", "issuing", "payments"]
    """
    Source balance type to pull funds from for the Balance Transfer.
    """


class BalanceTransferCreateParamsSourceBalanceAllocatedFunds(TypedDict):
    charge: str
    """
    The charge ID that the funds are originally sourced from. Required if `type` is `charge`.
    """
    type: Literal["charge"]
    """
    The type of object that the funds are originally sourced from. One of `charge`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.delegated_checkout._requested_session import RequestedSession
    from stripe.params.delegated_checkout._requested_session_confirm_params import (
        RequestedSessionConfirmParams,
    )
    from stripe.params.delegated_checkout._requested_session_create_params import (
        RequestedSessionCreateParams,
    )
    from stripe.params.delegated_checkout._requested_session_expire_params import (
        RequestedSessionExpireParams,
    )
    from stripe.params.delegated_checkout._requested_session_retrieve_params import (
        RequestedSessionRetrieveParams,
    )
    from stripe.params.delegated_checkout._requested_session_update_params import (
        RequestedSessionUpdateParams,
    )


class RequestedSessionService(StripeService):
    def retrieve(
        self,
        requested_session: str,
        params: Optional["RequestedSessionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RequestedSession":
        """
        Retrieves a requested session
        """
        return cast(
            "RequestedSession",
            self._request(
                "get",
                "/v1/delegated_checkout/requested_sessions/{requested_session}".format(
                    requested_session=sanitize_id(requested_session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        requested_session: str,
        params: Optional["RequestedSessionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RequestedSession":
        """
        Retrieves a requested session
        """
        return cast(
            "RequestedSession",
            await self._request_async(
                "get",
                "/v1/delegated_checkout/requested_sessions/{requested_session}".format(
                    requested_session=sanitize_id(requested_session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        requested_session: str,
        params: Optional["RequestedSessionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RequestedSession":
        """
        Updates a requested session
        """
        return cast(
            "RequestedSession",
            self._request(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}".format(
                    requested_session=sanitize_id(requested_session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        requested_session: str,
        params: Optional["RequestedSessionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RequestedSession":
        """
        Updates a requested session
        """
        return cast(
            "RequestedSession",
            await self._request_async(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}".format(
                    requested_session=sanitize_id(requested_session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "RequestedSessionCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "RequestedSession":
        """
        Creates a requested session
        """
        return cast(
            "RequestedSession",
            self._request(
                "post",
                "/v1/delegated_checkout/requested_sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "RequestedSessionCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "RequestedSession":
        """
        Creates a requested session
        """
        return cast(
            "RequestedSession",
            await self._request_async(
                "post",
                "/v1/delegated_checkout/requested_sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def confirm(
        self,
        requested_session: str,
        params: Optional["RequestedSessionConfirmParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RequestedSession":
        """
        Confirms a requested session
        """
        return cast(
            "RequestedSession",
            self._request(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/confirm".format(
                    requested_session=sanitize_id(requested_session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def confirm_async(
        self,
        requested_session: str,
        params: Optional["RequestedSessionConfirmParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RequestedSession":
        """
        Confirms a requested session
        """
        return cast(
            "RequestedSession",
            await self._request_async(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/confirm".format(
                    requested_session=sanitize_id(requested_session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def expire(
        self,
        requested_session: str,
        params: Optional["RequestedSessionExpireParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RequestedSession":
        """
        Expires a requested session
        """
        return cast(
            "RequestedSession",
            self._request(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/expire".format(
                    requested_session=sanitize_id(requested_session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def expire_async(
        self,
        requested_session: str,
        params: Optional["RequestedSessionExpireParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RequestedSession":
        """
        Expires a requested session
        """
        return cast(
            "RequestedSession",
            await self._request_async(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/expire".format(
                    requested_session=sanitize_id(requested_session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, Dict, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.entitlements._feature_create_params import (
        FeatureCreateParams,
    )
    from stripe.params.entitlements._feature_list_params import (
        FeatureListParams,
    )
    from stripe.params.entitlements._feature_modify_params import (
        FeatureModifyParams,
    )
    from stripe.params.entitlements._feature_retrieve_params import (
        FeatureRetrieveParams,
    )


class Feature(
    CreateableAPIResource["Feature"],
    ListableAPIResource["Feature"],
    UpdateableAPIResource["Feature"],
):
    """
    A feature represents a monetizable ability or functionality in your system.
    Features can be assigned to products, and when those products are purchased, Stripe will create an entitlement to the feature for the purchasing customer.
    """

    OBJECT_NAME: ClassVar[Literal["entitlements.feature"]] = (
        "entitlements.feature"
    )
    active: bool
    """
    Inactive features cannot be attached to new products and will not be returned from the features list endpoint.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    lookup_key: str
    """
    A unique key you provide as your own system identifier. This may be up to 80 characters.
    """
    metadata: Dict[str, str]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    name: str
    """
    The feature's name, for your own purpose, not meant to be displayable to the customer.
    """
    object: Literal["entitlements.feature"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """

    @classmethod
    def create(cls, **params: Unpack["FeatureCreateParams"]) -> "Feature":
        """
        Creates a feature
        """
        return cast(
            "Feature",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["FeatureCreateParams"]
    ) -> "Feature":
        """
        Creates a feature
        """
        return cast(
            "Feature",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["FeatureListParams"]
    ) -> ListObject["Feature"]:
        """
        Retrieve a list of features
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["FeatureListParams"]
    ) -> ListObject["Feature"]:
        """
        Retrieve a list of features
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["FeatureModifyParams"]
    ) -> "Feature":
        """
        Update a feature's metadata or permanently deactivate it.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Feature",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["FeatureModifyParams"]
    ) -> "Feature":
        """
        Update a feature's metadata or permanently deactivate it.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Feature",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["FeatureRetrieveParams"]
    ) -> "Feature":
        """
        Retrieves a feature
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["FeatureRetrieveParams"]
    ) -> "Feature":
        """
        Retrieves a feature
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.reporting._report_retrieve_params import (
        ReportRetrieveParams,
    )
    from stripe.v2.reporting._report import Report


class ReportService(StripeService):
    def retrieve(
        self,
        id: str,
        params: Optional["ReportRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Report":
        """
        Retrieves metadata about a specific `Report` template, including its name, description,
        and the parameters it accepts. It's useful for understanding the capabilities and
        requirements of a particular `Report` before requesting a `ReportRun`.
        """
        return cast(
            "Report",
            self._request(
                "get",
                "/v2/reporting/reports/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["ReportRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Report":
        """
        Retrieves metadata about a specific `Report` template, including its name, description,
        and the parameters it accepts. It's useful for understanding the capabilities and
        requirements of a particular `Report` before requesting a `ReportRun`.
        """
        return cast(
            "Report",
            await self._request_async(
                "get",
                "/v2/reporting/reports/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.billing.license_fees._version_list_params import (
        VersionListParams,
    )
    from stripe.params.v2.billing.license_fees._version_retrieve_params import (
        VersionRetrieveParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.billing._license_fee_version import LicenseFeeVersion


class VersionService(StripeService):
    def list(
        self,
        license_fee_id: str,
        params: Optional["VersionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[LicenseFeeVersion]":
        """
        List all versions of a License Fee object.
        """
        return cast(
            "ListObject[LicenseFeeVersion]",
            self._request(
                "get",
                "/v2/billing/license_fees/{license_fee_id}/versions".format(
                    license_fee_id=sanitize_id(license_fee_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        license_fee_id: str,
        params: Optional["VersionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[LicenseFeeVersion]":
        """
        List all versions of a License Fee object.
        """
        return cast(
            "ListObject[LicenseFeeVersion]",
            await self._request_async(
                "get",
                "/v2/billing/license_fees/{license_fee_id}/versions".format(
                    license_fee_id=sanitize_id(license_fee_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        license_fee_id: str,
        id: str,
        params: Optional["VersionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "LicenseFeeVersion":
        """
        Retrieve a License Fee Version object.
        """
        return cast(
            "LicenseFeeVersion",
            self._request(
                "get",
                "/v2/billing/license_fees/{license_fee_id}/versions/{id}".format(
                    license_fee_id=sanitize_id(license_fee_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        license_fee_id: str,
        id: str,
        params: Optional["VersionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "LicenseFeeVersion":
        """
        Retrieve a License Fee Version object.
        """
        return cast(
            "LicenseFeeVersion",
            await self._request_async(
                "get",
                "/v2/billing/license_fees/{license_fee_id}/versions/{id}".format(
                    license_fee_id=sanitize_id(license_fee_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )
